#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var inquirer = require("inquirer");
var fs = require("fs");
var path = require("path");
var shell = require("shelljs");
var chalk_1 = require("chalk");
var yargs = require("yargs");
/**
 * @description holds the project generator
 */
// Questions
var QUESTIONS = [
    {
        name: 'name',
        type: 'input',
        message: 'Project name:',
        when: function () { return !yargs.argv['name']; },
        validate: function (input) {
            if (/^([A-Za-z\-\_\d])+$/.test(input))
                return true;
            else
                return 'Project name may only include letters, numbers, underscores and hashes.';
        }
    }
];
// current directory
var CURR_DIR = process.cwd();
// prompts questions to user
inquirer.prompt(QUESTIONS)
    .then(function (answers) {
    var userAnswers = Object.assign({}, answers, yargs.argv);
    var projectName = userAnswers['name'];
    projectName = projectName.replace(/[^a-zA-Z]/g, "");
    var templatePath = path.join(__dirname, '../template');
    var targetPath = path.join(CURR_DIR);
    var templateConfig = getTemplateConfig(templatePath);
    var options = {
        projectName: projectName,
        templatePath: templatePath,
        targetPath: targetPath,
        config: templateConfig
    };
    if (!preProcess(options)) {
        return;
    }
    createDirectoryContents(templatePath, projectName, templateConfig);
    targetPath = path.join(CURR_DIR, projectName);
    options.targetPath = targetPath;
    if (!postProcess(options)) {
        return;
    }
    showMessage(options);
});
/**
 * shows message to user
 * @param options cli options
 */
var showMessage = function (options) {
    console.log('');
    console.log(chalk_1.default.green('Done.'));
    console.log(chalk_1.default.green("Go into the project: cd " + options.projectName));
    var message = options.config.postMessage;
    if (message) {
        console.log('');
        console.log(chalk_1.default.yellow(message));
        console.log('');
    }
};
/**
 * gets template config
 * @param templatePath template path
 * @returns tempate config
 */
var getTemplateConfig = function (templatePath) {
    var configPath = path.join(templatePath, '.template.json');
    if (!fs.existsSync(configPath))
        return {};
    var templateConfigContent = fs.readFileSync(configPath);
    if (templateConfigContent) {
        return JSON.parse(templateConfigContent.toString());
    }
    return {};
};
/**
 * applies post processes for node
 * @param options cli options
 */
var postProcess = function (options) {
    if (isNode(options)) {
        return postProcessNode(options);
    }
    return true;
};
/**
 * checks if package.json exists
 * @param options cli options
 * @returns true if package.json exists
 */
var isNode = function (options) {
    return fs.existsSync(path.join(options.templatePath, 'package.json'));
};
/**
 * applies post process for node,
 * npm install etc.
 * @param options cli options
 */
var postProcessNode = function (options) {
    shell.cd(options.targetPath);
    // cleaning unnecessary folder / file
    deleteDirectory(options.targetPath + "/template");
    fs.unlinkSync(options.targetPath + "/App.js");
    var cmd = 'npm install';
    if (cmd) {
        var result = shell.exec(cmd);
        if (result.code === 0) {
            cmd = "react-native link";
            var upgrade = shell.exec(cmd);
            if (upgrade.code !== 0) {
                return false;
            }
        }
        else {
            return false;
        }
    }
    else {
        console.log(chalk_1.default.red('No yarn or npm found. Cannot run installation.'));
    }
    return true;
};
var deleteDirectory = function (path) {
    if (fs.existsSync(path)) {
        fs.readdirSync(path).forEach(function (file, index) {
            var curPath = path + "/" + file;
            if (fs.lstatSync(curPath).isDirectory()) { // recurse
                deleteDirectory(curPath);
            }
            else { // delete file
                fs.unlinkSync(curPath);
            }
        });
        fs.rmdirSync(path);
    }
};
var preProcess = function (options) {
    shell.cd(options.targetPath);
    var cmd = 'npm i -g react-native-cli';
    if (cmd) {
        var result = shell.exec(cmd);
        if (result.code === 0) {
            cmd = "npx react-native init " + options.projectName + " --template typescript  --version react-native@0.61.5";
            var upgrade = shell.exec(cmd);
            if (upgrade.code !== 0) {
                return false;
            }
        }
        else {
            return false;
        }
    }
    else {
        console.log(chalk_1.default.red('No yarn or npm found. Cannot run installation.'));
    }
    return true;
};
// skip files
var SKIP_FILES = ['node_modules', '.template.json'];
/**
 * creates project directories with contents
 * @param templatePath template path
 * @param projectName project name
 * @param config template config
 */
var createDirectoryContents = function (templatePath, projectName, config) {
    var filesToCreate = fs.readdirSync(templatePath);
    filesToCreate.forEach(function (file) {
        var origFilePath = path.join(templatePath, file);
        // get stats about the current file
        var stats = fs.statSync(origFilePath);
        if (SKIP_FILES.indexOf(file) > -1)
            return;
        if (stats.isFile()) {
            var contents = fs.readFileSync(origFilePath, 'utf8');
            contents = contents.replace(/graphqlboilerplate/gi, projectName);
            var writePath = path.join(CURR_DIR, projectName, file);
            fs.writeFileSync(writePath, contents, 'utf8');
        }
        else if (stats.isDirectory()) {
            fs.mkdirSync(path.join(CURR_DIR, projectName, file));
            // recursive call
            createDirectoryContents(path.join(templatePath, file), path.join(projectName, file), config);
        }
    });
};
//# sourceMappingURL=index.js.map